'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.observe = observe;
exports.unobserve = unobserve;
if (global.window && !process.env.SERVER) {
  // eslint-disable-next-line global-require
  require('intersection-observer');
}

var INSTANCE_MAP = new Map();
var OBSERVER_MAP = new Map();

/**
 * Monitor element, and trigger callback when element becomes visible
 * @param element {HTMLElement}
 * @param callback {Function} - Called with inView
 * @param removeWhenVisible {Boolean} Removes the observer when it's triggered the first time.
 * @param threshold {Number} Number between 0 and 1, indicating how much of the element should be visible before triggering
 */
function observe(element, callback) {
  var removeWhenVisible = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
  var threshold = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;

  if (!element || !callback) return;
  var observerInstance = OBSERVER_MAP.get(threshold);
  if (!observerInstance) {
    observerInstance = new IntersectionObserver(onChange, { threshold: threshold });
    OBSERVER_MAP.set(threshold, observerInstance);
  }

  INSTANCE_MAP.set(element, {
    callback: callback,
    removeWhenVisible: removeWhenVisible,
    visible: false,
    threshold: threshold
  });
  observerInstance.observe(element);
}

/**
 * Stop observing an element. If an element is removed from the DOM or otherwise destroyed,
 * make sure to call this method. This is called automatically if an element has "removeWhenVisible" set to true.
 * @param element {HTMLElement}
 */
function unobserve(element) {
  if (!element) return;
  var instance = INSTANCE_MAP.get(element);

  if (instance) {
    INSTANCE_MAP.delete(element);

    var observerInstance = OBSERVER_MAP.get(instance.threshold);
    if (observerInstance) {
      observerInstance.unobserve(element);
    }

    var itemsLeft = Array.from(INSTANCE_MAP.values()).some(function (item) {
      return item.threshold === instance.threshold;
    });

    if (observerInstance && !itemsLeft) {
      // No more elements to observe, disconnect
      observerInstance.disconnect();
      OBSERVER_MAP.delete(instance.threshold);
    }
  }
}

function onChange(changes) {
  changes.forEach(function (intersection) {
    if (INSTANCE_MAP.has(intersection.target)) {
      var _INSTANCE_MAP$get = INSTANCE_MAP.get(intersection.target);

      var callback = _INSTANCE_MAP$get.callback;
      var removeWhenVisible = _INSTANCE_MAP$get.removeWhenVisible;
      var visible = _INSTANCE_MAP$get.visible;
      var threshold = _INSTANCE_MAP$get.threshold;
      // Trigger on 0 ratio only when not visible.

      var inView = visible ? intersection.intersectionRatio > threshold : intersection.intersectionRatio >= threshold;

      if (callback) {
        callback(inView);
      }

      // Remove element once it becomes visible
      if (removeWhenVisible) {
        unobserve(intersection.target);
      } else {
        INSTANCE_MAP.set(intersection.target, {
          callback: callback,
          removeWhenVisible: removeWhenVisible,
          visible: inView,
          threshold: threshold
        });
      }
    }
  });
}

exports.default = {
  observe: observe,
  unobserve: unobserve
};